/*
 * embapp.c - simple interactive query-issuing program, uses embedded server
 */

#include <string.h>		/* for strdup() */
/* #@ _HEADER_FILES_ */
#include <my_global.h>
#include <my_sys.h>
#include <mysql.h>
#include <mysql_embed.h>
#include <my_getopt.h>
/* #@ _HEADER_FILES_ */

static char *opt_host_name = NULL;		/* server host (default=localhost) */
static char *opt_user_name = NULL;		/* username (default=login name) */
static char *opt_password = NULL;		/* password (default=none) */
static unsigned int opt_port_num = 0;	/* port number (use built-in value) */
static char *opt_socket_name = NULL;	/* socket name (use built-in value) */
static char *opt_db_name = NULL;		/* database name (default=none) */
static unsigned int opt_flags = 0;		/* connection flags (none) */

static int ask_password = 0;			/* whether to solicit password */

static MYSQL *conn;						/* pointer to connection handler */

/* #@ _CLIENT_OPTION_GROUPS_ */
static const char *client_groups[] =
{
	"client", "embapp", NULL
};
/* #@ _CLIENT_OPTION_GROUPS_ */
/* #@ _SERVER_OPTION_GROUPS_ */
static const char *server_groups[] =
{
	"server", "embedded", "embapp_SERVER", NULL
};
/* #@ _SERVER_OPTION_GROUPS_ */

static struct my_option my_opts[] =		/* option information structures */
{
	{"help", '?', "Display this help and exit",
	NULL, NULL, NULL,
	GET_NO_ARG, NO_ARG, 0, 0, 0, 0, 0, 0},
	{"host", 'h', "Host to connect to",
	(gptr *) &opt_host_name, NULL, NULL,
	GET_STR_ALLOC, REQUIRED_ARG, 0, 0, 0, 0, 0, 0},
	{"password", 'p', "Password",
	(gptr *) &opt_password, NULL, NULL,
	GET_STR_ALLOC, OPT_ARG, 0, 0, 0, 0, 0, 0},
	{"port", 'P', "Port number",
	(gptr *) &opt_port_num, NULL, NULL,
	GET_UINT, REQUIRED_ARG, 0, 0, 0, 0, 0, 0},
	{"socket", 'S', "Socket path",
	(gptr *) &opt_socket_name, NULL, NULL,
	GET_STR_ALLOC, REQUIRED_ARG, 0, 0, 0, 0, 0, 0},
	{"user", 'u', "User name",
	(gptr *) &opt_user_name, NULL, NULL,
	GET_STR_ALLOC, REQUIRED_ARG, 0, 0, 0, 0, 0, 0},
	{ NULL, 0, NULL, NULL, NULL, NULL, GET_NO_ARG, NO_ARG, 0, 0, 0, 0, 0, 0 }
};

void
print_error (MYSQL *conn, char *message)
{
	fprintf (stderr, "%s\n", message);
	if (conn != NULL)
	{
		fprintf (stderr, "Error %u (%s)\n",
				mysql_errno (conn), mysql_error (conn));
	}
}

static my_bool
get_one_option (int optid, const struct my_option *opt, char *argument)
{
	switch (optid)
	{
	case '?':
		my_print_help (my_opts);	/* print help message */
		exit (0);
	case 'p':						/* password */
		if (!argument)				/* no value given, so solicit it later */
			ask_password = 1;
		else						/* copy password, wipe out original */
		{
			opt_password = strdup (argument);
			if (opt_password == NULL)
			{
				print_error (NULL, "could not allocate password buffer");
				exit (1);
			}
			while (*argument)
				*argument++ = 'x';
		}
		break;
	}
	return (0);
}

#include "process_result_set.c"

#include "process_query.c"

/* #@ _MAIN_ */
int
main (int argc, char *argv[])
{
int opt_err;

	my_init ();
	load_defaults ("my", client_groups, &argc, &argv);

	if ((opt_err = handle_options (&argc, &argv, my_opts, get_one_option)))
		exit (opt_err);

	/* solicit password if necessary */
	if (ask_password)
		opt_password = get_tty_password (NULL);

	/* get database name if present on command line */
	if (argc > 0)
	{
		opt_db_name = argv[0];
		--argc; ++argv;
	}

	/* initialize embedded server */
	mysql_server_init (0, NULL, (char **) server_groups);

	/* initialize connection handler */
	conn = mysql_init (NULL);
	if (conn == NULL)
	{
		print_error (NULL, "mysql_init() failed (probably out of memory)");
		exit (1);
	}

	/* connect to server */
	if (mysql_real_connect (conn, opt_host_name, opt_user_name, opt_password,
			opt_db_name, opt_port_num, opt_socket_name, opt_flags) == NULL)
	{
		print_error (conn, "mysql_real_connect() failed");
		mysql_close (conn);
		exit (1);
	}

	while (1)
	{
		char	buf[10000];

		fprintf (stderr, "query> ");					/* print prompt */
		if (fgets (buf, sizeof (buf), stdin) == NULL)	/* read query */
			break;
		if (strcmp (buf, "quit\n") == 0 || strcmp (buf, "\\q\n") == 0)
			break;
		process_query (conn, buf);						/* execute query */
	}

	/* disconnect from server */
	mysql_close (conn);
	/* shut down embedded server */
	mysql_server_end ();
	exit (0);
}
/* #@ _MAIN_ */
